
\name{e04bb}
\alias{e04bb}

\title{e04bb: Minimum, function of one variable, using first derivative}

\description{
e04bb searches for a minimum, in a given finite interval, of a continuous 
function of a single variable, using function and first derivative values. 
The method (based on cubic interpolation) is intended for functions which 
have a continuous first derivative (although it will usually work if the 
derivative has occasional discontinuities).



}

\usage{
e04bb(funct, e1, e2, a, b, maxcal)
}

\arguments{

\item{funct}{function

You must supply this function to calculate the values of \eqn{{F}\left({x}\right)}{F(x)} and \eqn{\frac{{{d}{F}}}{{{d}{x}}}}{(dF)/(dx)} at any point \eqn{{x}}{x} in \eqn{\left[{a}{b}\right]}{[ab]}.

\code{(FC,GC) = funct(xc)
}


}
\item{e1}{double

The relative accuracy to which the position of a minimum is required. (Note 
that, since e1 is a relative tolerance, the scaling of \eqn{{x}}{x} is 
automatically taken into account.)


}
\item{e2}{double

The absolute accuracy to which the position of a minimum is required. e2 
should be no smaller than \eqn{{2}{\epsilon}}{2\epsilon}.


}
\item{a}{double

The lower bound \eqn{{a}}{a} of the interval containing a minimum.


}
\item{b}{double

The upper bound \eqn{{b}}{b} of the interval containing a minimum.


}
\item{maxcal}{integer

The maximum number of calls of funct to be allowed.


}
}

\details{
R interface to the NAG Fortran routine E04BBF.}

\value{

\item{E1}{double

If you set e1 to \eqn{{0.0}}{0.0} (or to any value less than \eqn{{\epsilon}}{\epsilon}), 
e1 will be reset to the default value \eqn{\sqrt{{\epsilon}}}{sqrt(\epsilon)} before 
starting the minimization process.


}
\item{E2}{double

If you set e2 to \eqn{{0.0}}{0.0} (or to any value less than \eqn{{\epsilon}}{\epsilon}), 
e2 will be reset to the default value \eqn{\sqrt{{\epsilon}}}{sqrt(\epsilon)}.


}
\item{A}{double

An improved lower bound on the position of the minimum.


}
\item{B}{double

An improved upper bound on the position of the minimum.


}
\item{MAXCAL}{integer

The total number of times that funct was actually called.


}
\item{X}{double

The estimated position of the minimum.


}
\item{F}{double

The function value at the final point given in x.


}
\item{G}{double

The value of the first derivative at the final point in x.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04bbf.pdf}
}

\author{NAG}

\examples{

ifail <- 0
funct = function(xc) {
    
    fc <- sin(xc)/xc
    gc <- (cos(xc) - fc)/xc
    list(FC = fc, GC = gc)
}

e1 <- 0

e2 <- 0

a <- 3.5

b <- 5

maxcal <- 30

e04bb(funct, e1, e2, a, b, maxcal) 

}

\keyword{math}

\keyword{optimize}
\concept{derivative}
\concept{minimum}
