
\name{s17dl}
\alias{s17dl}

\title{s17dl: Hankel functions H_nu + a^(j)(z), j = 1 , 2, real a  >=  0, complex z, nu=0 , 1 , 2 ,  . . . }

\description{
s17dl returns a sequence of values for the Hankel functions \eqn{{H}_{{{\nu} 
+ {n}}}^{\left({1}\right)}\left({z}\right)}{H_\nu + n^(1)(z)} or \eqn{{H}_{{{\nu} 
+ {n}}}^{\left({2}\right)}\left({z}\right)}{H_\nu + n^(2)(z)} for complex 
\eqn{{z}}{z}, non-negative \eqn{{\nu}}{\nu} and \eqn{{n} = {0} , {1} , \ldots , 
{N} - {1}}{n = 0 , 1 ,  . . .  , N - 1}, with an option for exponential 
scaling.

}

\usage{
s17dl(m, fnu, z, n, scal)
}

\arguments{

\item{m}{integer

The kind of functions required. 

\eqn{m = {1}}{m = 1}: The functions are \eqn{{H}_{{\nu}}^{\left({1}\right)}\left({z}\right)}{H_\nu^(1)(z)}.

\eqn{m 
= {2}}{m = 2}: The functions are \eqn{{H}_{{\nu}}^{\left({2}\right)}\left({z}\right)}{H_\nu^(2)(z)}.




}
\item{fnu}{double

\eqn{{\nu}}{\nu}, the order of the first member of the sequence of functions.


}
\item{z}{complex

The argument \eqn{{z}}{z} of the functions.


}
\item{n}{integer

\eqn{{N}}{N}, the number of members required in the sequence \eqn{{H}_{{\nu}}^{\left(m\right)}\left({z}\right) 
, {H}_{{{\nu} + {1}}}^{\left(m\right)}\left({z}\right) , \ldots , {H}_{{{\nu} + 
{N} - {1}}}^{\left(m\right)}\left({z}\right)}{H_\nu^(m)(z) , H_\nu + 1^(m)(z) ,  . . .  , H_\nu + N - 1^(m)(z)}.


}
\item{scal}{string

The scaling option. 

\eqn{scal=\mathtt{'U'}}{scal='U'}: The results are returned unscaled.

\eqn{scal=\mathtt{'S'}}{scal='S'}: The results are returned scaled by the 
factor \eqn{{e}^{{ - {i}{z}}}}{e^ - iz} when \eqn{m = {1}}{m = 1}, or by 
the factor \eqn{{e}^{{{i}{z}}}}{e^iz} when \eqn{m = {2}}{m = 2}.




}
}

\details{
R interface to the NAG Fortran routine S17DLF.}

\value{

\item{CY}{complex array

The \eqn{{N}}{N} required function values: \eqn{cy[{i}]}{cy[i]} contains 
\eqn{{H}_{{{\nu} + {i} - {1}}}^{\left(m\right)}\left({z}\right)}{H_\nu + i - 1^(m)(z)} 
for \eqn{{i}={1} \ldots {N}}{i=1  . . .  N}.


}
\item{NZ}{integer

The number of components of cy that are set to zero due to underflow. If 
\eqn{nz > {0}}{nz > 0}, then if \eqn{\mathrm{Im}\left({z}\right) > {0.0}}{Im(z) > 0.0} 
and \eqn{m = {1}}{m = 1}, or \eqn{\mathrm{Im}\left({z}\right) < {0.0}}{Im(z) < 0.0} 
and \eqn{m = {2}}{m = 2}, elements \eqn{cy[{1}] , cy[{2}] , \ldots , cy[nz]}{cy[1] , cy[2] ,  . . .  , cy[nz]} 
are set to zero. In the complementary half-planes, nz simply states the 
number of underflows, and not which elements they are.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/S/s17dlf.pdf}
}

\author{NAG}

\examples{

ifail<-0

m<-1

fnu<-0

z<-complex(1,0.3,0.4)

n<-2

scal<-'U'

s17dl(m,fnu,z,n,scal)

}

\keyword{math}
\concept{hankel}
