
\name{s18gk}
\alias{s18gk}

\title{s18gk: Bessel function of the 1st kind J_alpha +/- n(z)}

\description{
s18gk returns a sequence of values for the Bessel functions \eqn{{J}_{{{\alpha} 
+ {n} - {1}}}\left({z}\right)}{J_\alpha + n - 1(z)} or \eqn{{J}_{{{\alpha} - {n} 
+ {1}}}\left({z}\right)}{J_\alpha - n + 1(z)} for complex \eqn{{z}}{z}, non-negative 
\eqn{{\alpha} < {1}}{\alpha < 1} and \eqn{{n} = {1} , {2} , \ldots , \mathrm{abs}({N}) 
+ {1}}{n = 1 , 2 ,  . . .  , abs(N) + 1}.

}

\usage{
s18gk(z, a, nl)
}

\arguments{

\item{z}{complex

The argument \eqn{{z}}{z} of the function.


}
\item{a}{double

The order \eqn{{\alpha}}{\alpha} of the first member in the required sequence of 
function values.


}
\item{nl}{integer

The value of \eqn{{N}}{N}.


}
}

\details{
R interface to the NAG Fortran routine S18GKF.}

\value{

\item{B}{complex array

With \eqn{\mathrm{ifail} =0}{ifail =0}, \eqn{\mathrm{ifail} =3}{ifail =3}, 
the required sequence of function values: \eqn{b[{n}]}{b[n]} contains \eqn{{J}_{{{\alpha} 
+ {n} - {1}}}\left({z}\right)}{J_\alpha + n - 1(z)} if \eqn{nl \geq {0}}{nl  >=  0} 
and \eqn{{J}_{{{\alpha} - {n} + {1}}}\left({z}\right)}{J_\alpha - n + 1(z)} otherwise 
for \eqn{{n}={1} \ldots \mathrm{abs}\left(nl\right)+{1}}{n=1  . . .  abs(nl)+1}.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/S/s18gkf.pdf}
}

\author{NAG}

\examples{

ifail<-0

z<-complex(1,0.6,-0.8)

a<-0

nl<-3

s18gk(z,a,nl)

}

\keyword{math}
\concept{Bessel}
