
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04jcf_OBJFUN(CONST Integer *N, CONST double X[],
                                   double *F, Integer *E04JCF_IUSER,
                                   double E04JCF_RUSER[],
                                   Integer *INFORM);
extern void NAG_CALL e04jcf_MONFUN(CONST Integer *N, CONST Integer *NF,
                                   CONST double X[], CONST double *F,
                                   CONST double *RHO, Integer *E04JCF_IUSER,
                                   double E04JCF_RUSER[],
                                   Integer *INFORM);

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04JCF_clean_up(double *X, double *BL, double *BU);

SEXP WE04JCF(SEXP OBJFUN, SEXP N, SEXP NPT, SEXP X, SEXP BL, SEXP BU,
             SEXP RHOBEG, SEXP RHOEND, SEXP MONFUN, SEXP MAXCAL,
             SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  Integer E04JCF_IUSER[6];
  double  E04JCF_RUSER[1];
  SEXP    ans, names;

/*initialization for input variables*/
  int     N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  int     NPT_integer;
  if (nag_extractInteger(NPT, &NPT_integer))
    {
      error("Integer extraction error for %s", "NPT");
    }
  double  *X_doubleMatrix = 0;
  double  *BL_doubleMatrix = 0;
  double  *BU_doubleMatrix = 0;
  double  RHOBEG_double;
  if (nag_extractDouble(RHOBEG, &RHOBEG_double))
    {
      error("Double extraction error for %s", "RHOBEG");
    }
  double  RHOEND_double;
  if (nag_extractDouble(RHOEND, &RHOEND_double))
    {
      error("Double extraction error for %s", "RHOEND");
    }
  int     MAXCAL_integer;
  if (nag_extractInteger(MAXCAL, &MAXCAL_integer))
    {
      error("Integer extraction error for %s", "MAXCAL");
    }

/*initialization for output variables*/
  double  F_double;
  int     NF_integer;

/*allocation general variables*/
  Integer X_dim1 = N_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }
  Integer BL_dim1 = N_integer;
  if (!(BL_doubleMatrix = (double *) malloc(BL_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "BL");
    }
  Integer BU_dim1 = N_integer;
  if (!(BU_doubleMatrix = (double *) malloc(BU_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "BU");
    }

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, X_dim1, 1, X, X_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "X", nrows(X), X_dim1);
        }
      else
        {
          E04JCF_clean_up(X_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix);
          error("Array extraction error for %s", "X");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, BL_dim1, 1, BL, BL_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "BL", nrows(BL), BL_dim1);
        }
      else
        {
          E04JCF_clean_up(X_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix);
          error("Array extraction error for %s", "BL");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, BU_dim1, 1, BU, BU_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "BU", nrows(BU), BU_dim1);
        }
      else
        {
          E04JCF_clean_up(X_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix);
          error("Array extraction error for %s", "BU");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Get the maximum number of iterations, or use default of 100 if not supplied */
  if (MAXCAL_integer == NA_INTEGER) MAXCAL_integer = 100;
  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04JCF_IUSER[0]);
  nag_convert_p2i(OBJFUN, &E04JCF_IUSER[2]);
  nag_convert_p2i(MONFUN, &E04JCF_IUSER[4]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04jcf_(e04jcf_OBJFUN, &N_integer, &NPT_integer, X_doubleMatrix,
          BL_doubleMatrix, BU_doubleMatrix, &RHOBEG_double, &RHOEND_double,
          e04jcf_MONFUN, &MAXCAL_integer, &F_double, &NF_integer, E04JCF_IUSER,
          E04JCF_RUSER,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04JCF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 4));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, X_dim1, 1, X_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_d2r(1, &F_double));
  SET_VECTOR_ELT(ans, 2, nag_i2r(1, &NF_integer));
  SET_VECTOR_ELT(ans, 3, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 4));
  SET_STRING_ELT(names, 0, mkChar("X"));
  SET_STRING_ELT(names, 1, mkChar("F"));
  SET_STRING_ELT(names, 2, mkChar("NF"));
  SET_STRING_ELT(names, 3, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04JCF_clean_up(X_doubleMatrix, BL_doubleMatrix, BU_doubleMatrix);

  return(ans);
}

void NAG_CALL e04jcf_OBJFUN(CONST Integer *N, CONST double X[], double *F,
                            Integer *E04JCF_IUSER, double E04JCF_RUSER[],
                            Integer *INFORM)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04JCF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04JCF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang3(
            funct,
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, X)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getDoubleScalarFromList("F", result, F);
  (void) nag_getIntFromList("INFORM", result, INFORM);

/* Release R_fcall */
  UNPROTECT(2);
}

void NAG_CALL e04jcf_MONFUN(CONST Integer *N, CONST Integer *NF,
                            CONST double X[], CONST double *F,
                            CONST double *RHO, Integer *E04JCF_IUSER,
                            double E04JCF_RUSER[],
                            Integer *INFORM)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04JCF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04JCF_IUSER[4], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang6(
            funct,
            nag_fi2r(1, N),
            nag_fi2r(1, NF),
            nag_md2r(1, *N, 1, X),
            nag_d2r(1, F),
            nag_d2r(1, RHO)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("INFORM", result, INFORM);

/* Release R_fcall */
  UNPROTECT(2);
}

void E04JCF_clean_up(double *X, double *BL, double *BU)
{
  if (X) free(X);
  if (BL) free(BL);
  if (BU) free(BU);
  return;
}
