#include "nag_rwrapper_utils.h"
extern double nag_getMaxValueDoubleVector(int n, double cm[])
{
  /* nag_getMaxValueDoubleVector: take a double precision vector and return the maximum value
     On entry:
     cm - Double vector
     n - length of the vector

     On exit:
     maxVal - the maximum value
   */
  int    i;
  double maxVal = cm[0];
  for (i = 0; i < n; i++)
    {
      if (cm[i] > maxVal)
        {
          maxVal = cm[i];
        }
    }

  return maxVal;
}

extern int nag_getMaxValueIntegerVector(int n, Integer cm[])
{
  /* nag_getMaxValueIntegerVector: take a double precision vector and return the maximum value
     On entry:
     cm - Integer vector
     n - length of the vector

     On exit:
     maxVal - the maximum value
   */
  int i;
  int maxVal = cm[0];
  for (i = 0; i < n; i++)
    {
      if (cm[i] > maxVal)
        {
          maxVal = cm[i];
        }
    }

  return maxVal;
}
int nag_extractDoubleVector(int n, SEXP v, double cv[])
{
  /* nag_extractDoubleVector: Take an R object and extract a double precision vector.

     On entry:
     n   - Length of the vector
     v   - R object holding the vector

     On exit:
     cv  - The vector

     Returns:
     error code, 0 = All OK, anything else implies an error in input
     vector, 1 = Not a vector, 2 = Not numeric, 3 = Length != n,
     4 = An entry was NaN or NA
   */
  int    failed, i;
  double *dv;
  SEXP   this_dv;

  if (!(failed = nag_isNumericVector(n, v)))
    {
      /* Cast the input vector into a vector of doubles */
      PROTECT(this_dv = coerceVector(v, REALSXP));

      /* Loop over each entry in v */
      dv = REAL(this_dv);
      for (i = 0; i < n; i++)
        {
          /* Check the entry is a valid double */
          if (!R_FINITE(dv[i]))
            {
              failed = 4;
              break;
            }

          /* Copy the entry into the output array */
          cv[i] = dv[i];
        }
      UNPROTECT(1);
    }

  return failed;
}
int nag_extractComplexVector(int n, SEXP v, Complex cv[])
{
  /* nag_extractComplexVector: Take an R object and extract a Complex vector.

     On entry:
     n   - Length of the vector
     v   - R object holding the vector

     On exit:
     cv  - The vector

     Returns:
     error code, 0 = All OK, anything else implies an error in input
     vector, 1 = Not a vector, 2 = Not numeric, 3 = Length != n,
     4 = An entry was NaN or NA
   */
  int      failed, i;
  Rcomplex *dv;
  SEXP     this_dv;

  failed = 0;
  /* Cast the input vector into a vector of R complex */
  PROTECT(this_dv = coerceVector(v, CPLXSXP));

  /* Loop over each entry in v */
  dv = COMPLEX(this_dv);
  for (i = 0; i < n; i++)
    {
      /* Copy the entry into the output array */
      cv[i].re = dv[i].r;
      cv[i].im = dv[i].i;
    }
  UNPROTECT(1);

  return failed;
}

int nag_copyDoubleVector(int nrows, int ncols, double *toBeCopied,
                         double **copy)
{
  /* nag_copyDoubleVector: allocates memory and generates a double vector that is a copy of the input vector
     On entry:
     toBeCopied   - the double vector to be copied
     copy   - the double  vector copy

     On exit:
     copy  - the double vector copy

   */

  int failed, i;

  if (!(*copy = (double *) malloc(nrows*ncols*sizeof(double))))
    {
      error("Memory allocation error while copying array");
    }

  for (i = 0; i < nrows*ncols; i++)
    {
      (*copy)[i] = toBeCopied[i];
    }
/*failed = memcpy(*copy, toBeCopied, nrows*ncols*sizeof(*toBeCopied)/sizeof(char));*/
  failed = 0;

  return failed;
}

int nag_copyNagFortranIntegerVector(int nrows, int ncols, Integer *toBeCopied,
                                    Integer **copy)
{
  /* nag_copyIntegerVector: allocates memory and generates an Integer vector that is a copy of the input vector
     On entry:
     toBeCopied   - the Integer vector to be copied
     copy   - the Integer  vector copy

     On exit:
     copy  - the Integer vector copy

   */

  int  failed, i;
  char EEE[200];

  if (!(*copy = (Integer *) malloc(nrows*ncols*sizeof(Integer))))
    {
      error("Memory allocation error while copying array");
    }
  sprintf(EEE, "BBB: %p  %p", *copy, copy);
  warning(EEE);

  failed = 1;
  for (i = 0; i < nrows*ncols; i++)
    {
      (*copy)[i] = toBeCopied[i];
    }
/*failed = memcpy(*copy, toBeCopied, nrows*ncols*sizeof(*toBeCopied)/sizeof(char));*/

  return failed;
}

int nag_extractNagFortranIntegerVector(int n, SEXP v, Integer cv[])
{
  /* nag_extractNagFortranIntegerVector: Take an R object and extract a Integer vector.

     On entry:
     n   - Length of the vector
     v   - R object holding the vector

     On exit:
     cv  - The vector

     Returns:
     error code, 0 = All OK, anything else implies an error in input
     vector, 1 = Not a vector, 2 = Not numeric, 3 = Length != n,
     4 = An entry was NaN or NA
   */
  int  failed, i;
  int  *iv;
  SEXP this_iv;

  if (!(failed = nag_isNumericVector(n, v)))
    {
      /* Cast the input vector into a vector of int's */
      PROTECT(this_iv = coerceVector(v, INTSXP));

      /* Loop over each entry in v */
      iv = INTEGER(this_iv);
      for (i = 0; i < n; i++)
        {
          /* Check the entry is a valid Integer */
          if (iv[i] == NA_INTEGER)
            {
              failed = 4;
              break;
            }
          /* Copy the entry into the output array */
          cv[i] = (Integer) iv[i];
        }
      UNPROTECT(1);
    }
  return failed;
}
int nag_extractLogicalVector(int n, SEXP v, Integer cv[])
{
  /* nag_extractLogicalVector: Take an R object and extract a Integer vector.

     On entry:
     n   - Length of the vector
     v   - R object holding the vector

     On exit:
     cv  - The vector

     Returns:
     error code, 0 = All OK, anything else implies an error in input
     vector, 1 = Not a vector, 2 = Not numeric, 3 = Length != n,
     4 = An entry was NaN or NA
   */
  int  failed, i;
  int  *iv;
  SEXP this_iv;

  if (!(failed = nag_isLogicalVector(n, v)))
    {
      /* Cast the input vector into a vector of logicals */
      PROTECT(this_iv = coerceVector(v, LGLSXP));
      /* Loop over each entry in v */
      iv = INTEGER(this_iv);
      for (i = 0; i < n; i++)
        {
          /* Check the entry is a valid Integer */
          if (iv[i] == NA_INTEGER)
            {
              failed = 4;
              break;
            }
          /* Copy the entry into the output array */

          cv[i] = (Integer) iv[i];
        }
      UNPROTECT(1);
    }

  return failed;
}
int nag_extractCharVector(int n, SEXP v, char cv[], int lstring)
{
  /* nag_extractCharVector: Take an R object and extract a char vector.

     On entry:
     n   - Length of the vector
     v   - R object holding the vector

     On exit:
     cv  - The vector

     Returns:
     error code, 0 = All OK, anything else implies an error in input
     vector, 1 = Not a vector, 2 = Not char, 3 = Length != n,
     4 = An entry was NaN or NA
   */
  int        failed, i, k, ii;
  const char *iv;
  SEXP       this_cv;

  if (!(failed = nag_isCharVector(n, v)))
    {
      /* Cast the input vector into a vector of logicals */
      PROTECT(this_cv = coerceVector(v, STRSXP));

      /* Loop over each entry in v */
      for (i = ii = 0; i < n; i++, ii += lstring)
        {
          iv = CHAR(STRING_ELT(this_cv, i));
          for (k = 0; *iv && k < lstring; iv++, k++)
            {
              cv[ii+k] = *iv;
            }
        }
      UNPROTECT(1);
    }

  return failed;
}

int nag_extractInteger(SEXP n, int *cn)
{
  /* nag_extractInteger: Take an R object and extract an int.

     On entry:
     n   - R object holding the vector

     On exit:
     cn  - The int

     Returns:
     error code, 0 = All OK, anything else implies an error in input
     object:   = An entry was NaN or NA
   */
  int failed = 0;
  *cn = INTEGER(n)[0];
  if (*cn == NA_INTEGER)
    {
      failed = 1;
    }

  return failed;
}

int nag_extractDouble(SEXP d, double *cd)
{
  /* nag_extractDouble: Take an R object and extract a double.

     On entry:
     n   - R object holding the vector

     On exit:
     cn  - The double

     Returns:
     error code, 0 = All OK, anything else implies an error in input
     object:  4 = An entry was NaN or NA
   */
  int failed = 0;

  *cd = REAL(d)[0];

  return failed;
}

int nag_extractComplex(SEXP c, Complex *cc)
{
  /* nag_extractComplex: Take an R object and extract a complex.

     On entry:
     c  - R object holding the vector

     On exit:
     cc  - The complex

     Returns:
     error code, 0 = All OK, anything else implies an error in input
     object:  4 = An entry was NaN or NA
   */
  int      failed = 0;
  Complex  cpltmp;
  Rcomplex cpl = COMPLEX(c)[0];

  cpltmp.re = cpl.r;
  cpltmp.im = cpl.i;

  *cc = cpltmp;

  return failed;
}

int nag_isNumericVector(int n, SEXP v)
{
  /* nag_isNumericVector: Checks that an R object is a numeric vector
     On entry:
     n   - Length of the vector
     v   - R object holding the vector

     Returns
     error code, 0 = All OK, anything else implies an error in input
     vector, 1 = Not a vector, 2 = Not numeric, 3 = Length != n
   */
  int failed = 0;

  if (!isVector(v))
    {
      failed = 1;
    }
  else if (!isNumeric(v))
    {
      failed = 2;
    }
  else if (LENGTH(v) != n)
    {
      failed = 3;
    }
  else
    {
      failed = 0;
    }

  return failed;
}
int nag_isLogicalVector(int n, SEXP v)
{
  /* nag_isLogicalVector: Checks that an R object is a numeric vector

     On entry:
     n   - Length of the vector
     v   - R object holding the vector

     Returns
     error code, 0 = All OK, anything else implies an error in input
     vector, 1 = Not a vector, 2 = Not numeric, 3 = Length != n
   */
  int failed = 0;

  if (!isVector(v))
    {
      failed = 1;
    }
  else if (!isNumeric(v))
    {
      failed = 2;
    }
  else if (LENGTH(v) != n)
    {
      failed = 3;
    }
  else
    {
      failed = 0;
    }

  return failed;
}

int nag_isCharVector(int n, SEXP v)
{
  /* nag_isCharVector: Checks that an R object is a character vector

     On entry:
     n   - Length of the vector
     v   - R object holding the vector

     Returns
     error code, 0 = All OK, anything else implies an error in input
     vector, 1 = Not a vector, 2 = Not numeric, 3 = Length != n
   */
  int failed = 0;

  if (!isVector(v))
    {
      failed = 1;
    }
  else if (!isString(v))
    {
      failed = 2;
    }
  else if (LENGTH(v) != n)
    {
      failed = 3;
    }
  else
    {
      failed = 0;
    }

  return failed;
}

int nag_initializeIntegerMatrix(int nrow, int ncol, Integer cm[])
{
  /* nag_initializeIntegerMatrix: set all elements of a double matrix to 0

     On entry:
     nrow   - Number of rows
     ncol   - Number of columns
     cm      - the matrix

     On exit:
     cm  - The matrix with all elements set to 0

   */
  int failed, i, j, k;
  failed = 0;
  /* Column major order (think this is the order R stores things in ???) */
  for (j = k = 0; j < ncol; j++)
    {
      for (i = 0; i < nrow; i++, k++)
        {
          cm[k] = 0;
        }
    }

  return failed;
}
int nag_initializeDoubleMatrix(int nrow, int ncol, double *cm)
{
  /* nag_initializeDoubleMatrix: set all elements of a double matrix to 0

     On entry:
     nrow   - Number of rows
     ncol   - Number of columns
     cm      - the matrix

     On exit:
     cm  - The matrix with all elements set to 0

   */
  int failed, i, j, k;
  failed = 0;
  /* Column major order (think this is the order R stores things in ???) */
  for (j = k = 0; j < ncol; j++)
    {
      for (i = 0; i < nrow; i++, k++)
        {
          cm[k] = 0;
        }
    }

  return failed;
}

int nag_extractDoubleMatrix(int cmajor, int nrow, int ncol, SEXP m, double *cm)
{
  /* nag_extractDoubleMatrix: Take an R object and extract a double precision matrix.

     On entry:
     cmajor - If set to 0 then the extracted matrix is in row major order,
              else it is in column major order
     nrow   - Number of rows
     ncol   - Number of columns
     m      - R object holding the matrix

     On exit:
     cm  - The matrix

     Returns:
     error code, 0 = All OK, anything else implies an error in input
     vector, 1 = Not a matrix, 2 = Not numeric, 3 = Number of rows != nrow,
     4 = Number of columns != ncol, 5 = An entry was NaN or NA
   */
  int    failed, i, j, k, pk;
  double *dm;

  if (!(failed =
          nag_isNumericMatrix(nrow, ncol, m)) || failed == 3 || failed == 4)
    {
      dm = REAL(m);
      if (cmajor)
        {
          /* Column major order (think this is the order R stores things in ???) */
          for (j = k = 0; j < ncol; j++)
            {
              for (i = 0; i < nrow; i++, k++)
                {
                  if (!R_FINITE(dm[k]))
                    {
                      failed = 5;
                      break;
                    }
                  cm[k] = dm[k];
                }
            }
        }
      else
        {
          /* Row major order */
          for (j = k = 0; j < ncol; j++)
            {
              for (i = 0; i < nrow; i++, k++)
                {
                  if (!R_FINITE(dm[k]))
                    {
                      failed = 5;
                      break;
                    }
                  pk = i * ncol + j;
                  cm[pk] = dm[k];
                }
            }
        }
    }

  return failed;
}

int nag_extractIntegerMatrix(int cmajor, int nrow, int ncol, SEXP m,
                             Integer cm[])
{
  /* nag_extractIntegerMatrix: Take an R object and extract a Integer matrix.

     On entry:
     cmajor - If set to 0 then the extracted matrix is in row major order,
        else it is in column major order
     nrow   - Number of rows
     ncol   - Number of columns
     m      - R object holding the matrix

     On exit:
     cm  - The matrix

     Returns:
     error code, 0 = All OK, anything else implies an error in input
     vector, 1 = Not a matrix, 2 = Not numeric, 3 = Number of rows != nrow,
     4 = Number of columns != ncol, 5 = An entry was NaN or NA
   */
  int  failed, i, j, k, pk;
  int  *im;
  SEXP m_this;

  if ((!(failed =
           nag_isIntegerMatrix(nrow, ncol,
                               m))) || (failed == 3) || (failed = 4))
    {
      PROTECT(m_this = coerceVector(m, INTSXP));
      im = INTEGER(m_this);
      if (cmajor)
        {
          /* Column major order (think this is the order R stores things in ???) */
          for (j = k = 0; j < ncol; j++)
            {
              for (i = 0; i < nrow; i++, k++)
                {
                  if (im[k] == NA_INTEGER)
                    {
                      failed = 5;
                      break;
                    }
                  cm[k] = (Integer) im[k];
                }
            }
        }
      else
        {
          /* Row major order */
          for (j = k = 0; j < ncol; j++)
            {
              for (i = 0; i < nrow; i++, k++)
                {
                  if (im[k] == NA_INTEGER)
                    {
                      failed = 5;
                      break;
                    }
                  pk = i * ncol + j;
                  cm[pk] = (Integer) im[k];
                }
            }
        }
      UNPROTECT(1);
    }

  return failed;
}

int nag_isNumericMatrix(int nrow, int ncol, SEXP m)
{
  /* nag_isNumericMatrix: Take an R object and extract a double precision matrix.

     On entry:
     nrow   - Number of rows
     ncol   - Number of columns

     Returns:
     error code, 0 = All OK, anything else implies an error in input
     vector, 1 = Not a matrix, 2 = Not numeric, 3 = Number of rows != nrow,
     4 = Number of columns != ncol
   */
  int failed = 0;

  if (!isMatrix(m))
    {
      failed = 1;
    }
  else if (!isNumeric(m))
    {
      failed = 2;
    }
  else if (nrows(m) != nrow)
    {
      failed = 3;
    }
  else if (ncols(m) != ncol)
    {
      failed = 4;
    }
  else
    {
      failed = 0;
    }

  return failed;
}

int nag_isIntegerMatrix(int nrow, int ncol, SEXP m)
{
  /* nag_isIntegerMatrix: Take an R object and checks if it is a Integer matrix.

     On entry:
     nrow   - Number of rows
     ncol   - Number of columns

     Returns:
     error code, 0 = All OK, anything else implies an error in input
     vector, 1 = Not a matrix, 2 = Not numeric, 3 = Number of rows != nrow,
     4 = Number of columns != ncol
   */
  int failed = 0;

  if (!isMatrix(m))
    {
      failed = 1;
    }
  else if (!isNumeric(m))
    {
      failed = 2;
    }
  else if (nrows(m) != nrow)
    {
      failed = 3;
    }
  else if (ncols(m) != ncol)
    {
      failed = 4;
    }
  else
    {
      failed = 0;
    }

  return failed;
}

SEXP nag_getListElement(SEXP list, const char *str)
{
  /* nag_getListElement: Returns the list element named str, or returns NULL.

     This is based on the function getListElement in "Writing R Extensions"
   */

  SEXP elmt, names;
  int  i;

  elmt = R_NilValue;
  names = getAttrib(list, R_NamesSymbol);

  for (i = 0; i < length(list); i++)
    {
      if (!strcmp(CHAR(STRING_ELT(names, i)), str))
        {
          elmt = VECTOR_ELT(list, i);
          break;
        }
    }
  return elmt;
}

SEXP nag_md2r(int cmajor, int nrow, int ncol, CONST double m[])
{
  /* nag_md2r: Take a matrix of doubles and return an matrix R object
     cmajor - If 0 then matrix m is in row major order, else m is in
              column major order
     nrow   - Number of rows
     ncol   - Number of columns
     m      - Array of doubles
   */
  SEXP   ans;
  double *dans;
  int    i, j, k, pk;

  /* Protect the vector ans from R's garbage collector */
  PROTECT(ans = allocMatrix(REALSXP, nrow, ncol));
  dans = (double *) REAL(ans);
  if (cmajor)
    {
      for (j = k = 0; j < ncol; j++)
        {
          for (i = 0; i < nrow; i++, k++)
            {
              dans[k] = m[k];
            }
        }
    }
  else
    {
      for (j = k = 0; j < ncol; j++)
        {
          for (i = 0; i < nrow; i++, k++)
            {
              pk = i * ncol + j;
              dans[k] = m[pk];
            }
        }
    }

  /* Unprotect the last 1 object protected (which in this is ans) */
  UNPROTECT(1);

  return ans;
}

SEXP nag_mi2r(int cmajor, int nrow, int ncol, Integer m[])
{
  /* nag_mi2r: Take a matrix of Integer and return an matrix R object
     cmajor - If 0 then matrix m is in row major order, else m is in
              column major order
     nrow   - Number of rows
     ncol   - Number of columns
     m      - Array of doubles
   */
  SEXP    ans;
  Integer *dans;
  int     i, j, k, pk;

  /* Protect the vector ans from R's garbage collector */
  PROTECT(ans = allocMatrix(INTSXP, nrow, ncol));
  dans = (Integer *) INTEGER(ans);
  if (cmajor)
    {
      for (j = k = 0; j < ncol; j++)
        {
          for (i = 0; i < nrow; i++, k++)
            {
              dans[k] = m[k];
            }
        }
    }
  else
    {
      for (j = k = 0; j < ncol; j++)
        {
          for (i = 0; i < nrow; i++, k++)
            {
              pk = i * ncol + j;
              dans[k] = m[pk];
            }
        }
    }

  /* Unprotect the last 1 object protected (which in this is ans) */
  UNPROTECT(1);

  return ans;
}

SEXP nag_i2r(int n, Integer x[])
{
  /* nag_i2r: Take an array of Integers, and return an INTEGER R object
     n  - Number of elements
     x  - Array of doubles
   */
  SEXP ans;
  int  *ians, i;

  /* Protect the vector ans from R's garbage collector */
  PROTECT(ans = allocVector(INTSXP, n));
  ians = INTEGER(ans);
  for (i = 0; i < n; i++)
    {
      ians[i] = (int) x[i];
    }

  /* Unprotect the last 1 object protected (which in this is ans) */
  UNPROTECT(1);

  return ans;
}

SEXP nag_fi2r(int n, CONST Integer x[])
{
  /* nag_fi2r: Take an array of Integer, and return an INTEGER R object
     n  - Number of elements
     x  - Array of integers
   */
  SEXP ans;
  int  *ians, i;

  /* Protect the vector ans from R's garbage collector */
  PROTECT(ans = allocVector(INTSXP, n));
  ians = INTEGER(ans);
  for (i = 0; i < n; i++)
    {
      ians[i] = (int) x[i];
    }

  /* Unprotect the last 1 object protected (which in this is ans) */
  UNPROTECT(1);

  return ans;
}
SEXP nag_l2r(int n, Integer x[])
{
  /* nag_l2r: Take an array of Integer, and return a LOGICAL R object
     n  - Number of elements
     x  - Array of logicals
   */
  SEXP ans;
  int  *ians, i;

  /* Protect the vector ans from R's garbage collector */
  PROTECT(ans = allocVector(LGLSXP, n));
  ians = LOGICAL(ans);
  for (i = 0; i < n; i++)
    {
      ians[i] = (int) x[i];
    }

  /* Unprotect the last 1 object protected (which in this is ans) */
  UNPROTECT(1);

  return ans;
}
SEXP nag_c2r(int n, int lstring, int trim, char x[])
{
  /* nag_c2r: Take an array of char, and return a char R object
     n  - Number of elements
     x  - Array of char
   */
  SEXP ans;
  /**ians, i;*/
  int  i, k, j;
  char *px;

  px = (char *) malloc((lstring+1)*sizeof(char));

  /* Protect the vector ans from R's garbage collector */
  PROTECT(ans = allocVector(STRSXP, n));
  /*ians = CHAR(ans);*/
  for (i = k = 0; i < n; i++)
    {
      for (j = 0; j < lstring; j++, k++)
        {
          px[j] = x[k];
        }
      if (trim)
        {
          for (j--; j > 0 && px[j] == ' '; j--) ;
          px[j+1] = '\0';
        }
      else
        {
          px[lstring] = '\0';
        }
      SET_STRING_ELT(ans, i, mkChar(px));
    }

  /* Unprotect the last 1 object protected (which in this is ans) */
  UNPROTECT(1);

  free(px);

  return ans;
}
SEXP nag_l2l(int n, Nag_Boolean x[])
{
  /* nag_l2l: Take an array of Nag_Boolean, and return a LGLSXP R object
     n  - Number of elements
     x  - Array of doubles
   */
  SEXP ans;
  int  *ians;
  int  i;

  /* Protect the vector ans from R's garbage collector */
  PROTECT(ans = allocVector(LGLSXP, n));
  for (i = 0; i < n; i++)
    {
      ians = LOGICAL(ans);
      if (x[i] == Nag_TRUE)
        {
          ians[i] = 1;
        }
      else
        {
          ians[i] = 0;
        }
    }

  /* Unprotect the last 1 object protected (which in this is ans) */
  UNPROTECT(1);

  return ans;
}

SEXP nag_d2r(int n, CONST double x[])
{
  /* nag_d2r: Take an array of doubles, and return a REAL R object
     n  - Number of elements
     x  - Array of doubles
   */
  SEXP   ans;
  double *rans;
  int    i;

  /* Protect the vector ans from R's garbage collector */
  PROTECT(ans = allocVector(REALSXP, n));
  rans = REAL(ans);
  for (i = 0; i < n; i++)
    {
      rans[i] = x[i];
    }

  /* Unprotect the last 1 object protected (which in this is ans) */
  UNPROTECT(1);

  return ans;
}

SEXP nag_cplx2r(int n, CONST Complex x[])
{
  /* nag_cplx2r: Take an array of complexes, and return a CPLX R object
     n  - Number of elements
     x  - Array of complexes
   */
  SEXP     ans;
  Rcomplex *rans;

  int      i;

  /* Protect the vector ans from R's garbage collector */
  PROTECT(ans = allocVector(CPLXSXP, n));
  rans = COMPLEX(ans);
  for (i = 0; i < n; i++)
    {
      rans[i].r = x[i].re;
      rans[i].i = x[i].im;
    }

  /* Unprotect the last 1 object protected (which in this is ans) */
  UNPROTECT(1);

  return ans;
}

SEXP nag_2d2r(int n1, double x1[], int n2, double x2[])
{
  /* nag_d2r: Take an two arrays of doubles, and return a REAL R object
     n1 - Number of elements in first array
     x1 - First array of doubles
     n2 - Number of elements in second array
     x2 - Second array of doubles

     Returns and REAL vector equivalent to c(x1,x2)
   */
  SEXP   ans;
  double *rans;
  int    i, j;

  /* Protect the vector ans from R's garbage collector */
  PROTECT(ans = allocVector(REALSXP, n1 + n2));
  rans = REAL(ans);
  for (j = 0; j < n1; j++)
    {
      rans[j] = x1[j];
    }
  for (i = 0; i < n2; i++, j++)
    {
      rans[j] = x2[i];
    }

  /* Unprotect the last 1 object protected (which in this is ans) */
  UNPROTECT(1);

  return ans;
}

/*************************************************************************************/
/* Routines for storing and retrieving a pointer in an array of integerrs */
#ifdef WIN64
#define MY_NAG_ULONG_TYPE unsigned __int64
#else
#define MY_NAG_ULONG_TYPE unsigned long
#endif

void nag_convert_p2i(SEXP p, Integer *v)
{
  /* NAG_CONVERT_P2I: Convert an SEXP object into two integers.
     On entry:
     p - Pointer to SEXP object

     On exit:
     v - Array to hold the converted pointer, v[2]
   */
  MY_NAG_ULONG_TYPE lp;

  lp = (MY_NAG_ULONG_TYPE) p;

#ifdef IS_64BIT
  v[0] = (Integer) (lp >> 32);
  v[1] = (Integer) (lp & 0xffffffff);
#else
  /* Handle 32-bit case separately just to avoid gcc compiler
     warning about shifting a 32-bit quantity by 32 bits */
  v[0] = 0;
  v[1] = (Integer) lp;
#endif
}

void nag_convert_i2p(Integer *v, SEXP *p)
{
  /* NAG_CONVERT_I2P: Convert two integers into an SEXP object
     On entry:
     v - Array to holding the converted pointer, v[2]

     On exit:
     p - Pointer to function
   */
  MY_NAG_ULONG_TYPE lp;

  if (v[0] != 0 || v[1] != 0)
    {
#ifdef IS_64BIT
      /* N.B. weird casting of v[1] to acml_uint_t is because without it,
         if the top bit of v[1] happens to be set, a cast directly to unsigned
         long will propagate the 1 bit all the way to the 64th bit */
      lp =
        ((MY_NAG_ULONG_TYPE) v[0] <<
         32) | ((MY_NAG_ULONG_TYPE) ((Integer) v[1]));
      *p = (SEXP) lp;
#else
      /* Handle 32-bit case separately just to avoid gcc compiler
         warning about shifting a 32-bit quantity by 32 bits */
      lp = (MY_NAG_ULONG_TYPE) v[1];
      *p = (SEXP) lp;
#endif
    }
  else
    *p = NULL;
}

#undef MY_NAG_ULONG_TYPE
/* Routines for storing and retrieving a pointer in an array of integerrs */
/*************************************************************************************/

/*************************************************************************************/
/* The following functions are extensions of the lang2 to lang4 and list1 to list 4
   in the Rinlinedfuns.h.

   These are only needed when use callback functions that are the function themselves
   (rather than the body of the function). Probably better to just stick with using
   the body of the function!
 */
SEXP lang5(SEXP s, SEXP p1, SEXP p2, SEXP p3, SEXP p4)
{
  PROTECT(s);
  s = LCONS(s, list4(p1, p2, p3, p4));
  UNPROTECT(1);
  return s;
}

SEXP lang6(SEXP s, SEXP p1, SEXP p2, SEXP p3, SEXP p4, SEXP p5)
{
  PROTECT(s);
  s = LCONS(s, list5(p1, p2, p3, p4, p5));
  UNPROTECT(1);
  return s;
}

SEXP lang7(SEXP s, SEXP p1, SEXP p2, SEXP p3, SEXP p4, SEXP p5, SEXP p6)
{
  PROTECT(s);
  s = LCONS(s, list6(p1, p2, p3, p4, p5, p6));
  UNPROTECT(1);
  return s;
}

SEXP lang8(SEXP s, SEXP p1, SEXP p2, SEXP p3, SEXP p4, SEXP p5, SEXP p6,
           SEXP p7)
{
  PROTECT(s);
  s = LCONS(s, list7(p1, p2, p3, p4, p5, p6, p7));
  UNPROTECT(1);
  return s;
}

SEXP lang9(SEXP s, SEXP p1, SEXP p2, SEXP p3, SEXP p4, SEXP p5, SEXP p6,
           SEXP p7,
           SEXP p8)
{
  PROTECT(s);
  s = LCONS(s, list8(p1, p2, p3, p4, p5, p6, p7, p8));
  UNPROTECT(1);
  return s;
}

SEXP lang10(SEXP s, SEXP p1, SEXP p2, SEXP p3, SEXP p4, SEXP p5, SEXP p6,
            SEXP p7, SEXP p8,
            SEXP p9)
{
  PROTECT(s);
  s = LCONS(s, list9(p1, p2, p3, p4, p5, p6, p7, p8, p9));
  UNPROTECT(1);
  return s;
}

SEXP lang11(SEXP s, SEXP p1, SEXP p2, SEXP p3, SEXP p4, SEXP p5, SEXP p6,
            SEXP p7, SEXP p8, SEXP p9,
            SEXP p10)
{
  PROTECT(s);
  s = LCONS(s, list10(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10));
  UNPROTECT(1);
  return s;
}

SEXP lang12(SEXP s, SEXP p1, SEXP p2, SEXP p3, SEXP p4, SEXP p5, SEXP p6,
            SEXP p7, SEXP p8, SEXP p9, SEXP p10,
            SEXP p11)
{
  PROTECT(s);
  s = LCONS(s, list11(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11));
  UNPROTECT(1);
  return s;
}

SEXP lang13(SEXP s, SEXP p1, SEXP p2, SEXP p3, SEXP p4, SEXP p5, SEXP p6,
            SEXP p7, SEXP p8, SEXP p9, SEXP p10, SEXP p11,
            SEXP p12)
{
  PROTECT(s);
  s = LCONS(s, list12(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12));
  UNPROTECT(1);
  return s;
}
SEXP lang14(SEXP s, SEXP p1, SEXP p2, SEXP p3, SEXP p4, SEXP p5, SEXP p6,
            SEXP p7, SEXP p8, SEXP p9, SEXP p10, SEXP p11, SEXP p12, SEXP p13)
{
  PROTECT(s);
  s =
    LCONS(s,
          list13(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13));
  UNPROTECT(1);
  return s;
}
SEXP lang15(SEXP s, SEXP p1, SEXP p2, SEXP p3, SEXP p4, SEXP p5, SEXP p6,
            SEXP p7, SEXP p8, SEXP p9, SEXP p10, SEXP p11, SEXP p12, SEXP p13,
            SEXP p14)
{
  PROTECT(s);
  s =
    LCONS(s,
          list14(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14));
  UNPROTECT(1);
  return s;
}

SEXP list5(SEXP s, SEXP l1, SEXP l2, SEXP l3, SEXP l4)
{
  PROTECT(s);
  s = CONS(s, list4(l1, l2, l3, l4));
  UNPROTECT(1);
  return s;
}

SEXP list6(SEXP s, SEXP l1, SEXP l2, SEXP l3, SEXP l4, SEXP l5)
{
  PROTECT(s);
  s = CONS(s, list5(l1, l2, l3, l4, l5));
  UNPROTECT(1);
  return s;
}

SEXP list7(SEXP s, SEXP l1, SEXP l2, SEXP l3, SEXP l4, SEXP l5, SEXP l6)
{
  PROTECT(s);
  s = CONS(s, list6(l1, l2, l3, l4, l5, l6));
  UNPROTECT(1);
  return s;
}

SEXP list8(SEXP s, SEXP l1, SEXP l2, SEXP l3, SEXP l4, SEXP l5, SEXP l6,
           SEXP l7)
{
  PROTECT(s);
  s = CONS(s, list7(l1, l2, l3, l4, l5, l6, l7));
  UNPROTECT(1);
  return s;
}

SEXP list9(SEXP s, SEXP l1, SEXP l2, SEXP l3, SEXP l4, SEXP l5, SEXP l6,
           SEXP l7,
           SEXP l8)
{
  PROTECT(s);
  s = CONS(s, list8(l1, l2, l3, l4, l5, l6, l7, l8));
  UNPROTECT(1);
  return s;
}

SEXP list10(SEXP s, SEXP l1, SEXP l2, SEXP l3, SEXP l4, SEXP l5, SEXP l6,
            SEXP l7, SEXP l8,
            SEXP l9)
{
  PROTECT(s);
  s = CONS(s, list9(l1, l2, l3, l4, l5, l6, l7, l8, l9));
  UNPROTECT(1);
  return s;
}

SEXP list11(SEXP s, SEXP l1, SEXP l2, SEXP l3, SEXP l4, SEXP l5, SEXP l6,
            SEXP l7, SEXP l8, SEXP l9,
            SEXP l10)
{
  PROTECT(s);
  s = CONS(s, list10(l1, l2, l3, l4, l5, l6, l7, l8, l9, l10));
  UNPROTECT(1);
  return s;
}

SEXP list12(SEXP s, SEXP l1, SEXP l2, SEXP l3, SEXP l4, SEXP l5, SEXP l6,
            SEXP l7, SEXP l8, SEXP l9, SEXP l10,
            SEXP l11)
{
  PROTECT(s);
  s = CONS(s, list11(l1, l2, l3, l4, l5, l6, l7, l8, l9, l10, l11));
  UNPROTECT(1);
  return s;
}
SEXP list13(SEXP s, SEXP l1, SEXP l2, SEXP l3, SEXP l4, SEXP l5, SEXP l6,
            SEXP l7, SEXP l8, SEXP l9, SEXP l10, SEXP l11,
            SEXP l12)
{
  PROTECT(s);
  s = CONS(s, list12(l1, l2, l3, l4, l5, l6, l7, l8, l9, l10, l11, l12));
  UNPROTECT(1);
  return s;
}
SEXP list14(SEXP s, SEXP l1, SEXP l2, SEXP l3, SEXP l4, SEXP l5, SEXP l6,
            SEXP l7, SEXP l8, SEXP l9, SEXP l10, SEXP l11, SEXP l12,
            SEXP l13)
{
  PROTECT(s);
  s = CONS(s, list13(l1, l2, l3, l4, l5, l6, l7, l8, l9, l10, l11, l12, l13));
  UNPROTECT(1);
  return s;
}
SEXP list15(SEXP s, SEXP l1, SEXP l2, SEXP l3, SEXP l4, SEXP l5, SEXP l6,
            SEXP l7, SEXP l8, SEXP l9, SEXP l10, SEXP l11, SEXP l12, SEXP l13,
            SEXP l14)
{
  PROTECT(s);
  s =
    CONS(s, list14(l1, l2, l3, l4, l5, l6, l7, l8, l9, l10, l11, l12, l13, l14));
  UNPROTECT(1);
  return s;
}

int nag_getDoubleScalarFromList(const char *name, SEXP result, double *pvalue)
{
  SEXP   list_value;
  int    valid;
  double value;

  /* assume that result is going to be invalid */
  valid = 0;
  *pvalue = 0.0;

  /* search list for required value */
  PROTECT(list_value = nag_getListElement(result, name));

  if (list_value != R_NilValue)
    {
      /* if value exists then try and extract it */
      value = REAL(list_value)[0];
      if (R_FINITE(value))
        {
          /* all is OK */
          *pvalue = value;
          valid = 1;
        }
    }
  UNPROTECT(1);

  return valid;
}
int nag_getIntFromList(const char *name, SEXP result, Integer *pvalue)
{
  SEXP list_value;
  int  valid;
  int  value;

  /* assume that result is going to be invalid */
  valid = 0;
  *pvalue = 0;

  /* search list for required value */
  PROTECT(list_value = nag_getListElement(result, name));

  if (list_value != R_NilValue)
    {
      /* if value exists then try and extract it */
      value = INTEGER(list_value)[0];
      if (R_FINITE(value))
        {
          /* all is OK */
          *pvalue = value;
          valid = 1;
        }
    }
  UNPROTECT(1);

  return valid;
}
int nag_getDoubleMatrixFromList(const char *name, int cmajor, int nrows,
                                int ncols, SEXP result,
                                double rvector[])
{
  SEXP tgrad;
  int  valid;

/* assume that result is going to be invalid */
  valid = 0;

  PROTECT(tgrad = nag_getListElement(result, name));

  if (tgrad != R_NilValue)
    {
      valid = nag_extractDoubleMatrix(cmajor, nrows, ncols, tgrad, rvector);
    }
  else
    {
      error("Expected object not found for %s", name);
    }
  UNPROTECT(1);

  return valid;
}
int nag_getIntegerMatrixFromList(const char *name, int cmajor, int nrows,
                                 int ncols, SEXP result,
                                 Integer ivector[])
{
  SEXP tgrad;
  int  valid;

/* assume that result is going to be invalid */
  valid = 0;

  PROTECT(tgrad = nag_getListElement(result, name));

  if (tgrad != R_NilValue)
    {
      valid = nag_extractIntegerMatrix(cmajor, nrows, ncols, tgrad, ivector);
    }
  UNPROTECT(1);

  return valid;
}

int nag_maxCharLenInCharVector(int n, SEXP v)
{
  SEXP       this_cv;
  int        i, this_len, max_len;
  const char *iv;

  if (!nag_isCharVector(n, v))
    {
      PROTECT(this_cv = coerceVector(v, STRSXP));

      /* number of elements in vector */
      n = LENGTH(v);

      /* initialise maximum length */
      max_len = 0;

      /* loop over each element in the character vector */
      for (i = 0; i < n; i++)
        {
          iv = CHAR(STRING_ELT(this_cv, i));
          for (this_len = 0; *iv; iv++, this_len++) ;
          max_len = (this_len > max_len)?this_len:max_len;
        }

      UNPROTECT(1);
    }
  else
    {
      max_len = 0;
    }

  return max_len;
}
/*************************************************************************************/

